﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Leap;

namespace ARDroneListener
{
    public class ARListener : Listener
    {
        private object verrou = new object();
        private int history = 5;
        private int nbUp = 0;
        private int nbDown = 0;
        private long timeStampTakeOff;
        private long timeStampLanding;
        private bool isFlying;
        private bool isVerticalGesture;

        private float? previousPitch;
        private float? previousRoll;
        private float? previousYaw;

        private DateTime? lastNotification;
        public override void OnFrame(Controller controller)
        {
            var oldFrame = controller.Frame(history);
            
            if (!oldFrame.IsValid) return;
            isVerticalGesture = false;
            var currentFrame = controller.Frame();
            if (currentFrame.IsValid)
            {
                if (currentFrame.Hands.Count > 0)
                {
                    var hand = currentFrame.Hands[0];
                    var translation = hand.Translation(oldFrame);
                    //Console.WriteLine("Translation : {0}", translation);
                    if (translation.Magnitude > 50)
                    {
                        if (translation.y < -100)
                        {
                            lock (verrou)
                            {
                                nbDown++;    
                            }
                        }
                        else
                        {
                            if (translation.y > 0)
                            {
                                lock (verrou)
                                {
                                    nbUp++;
                                }
                            }
                        }    
                    }

                    if (nbUp > 5)
                    {
                        lock (verrou)
                        {
                            nbUp = 0;
                            nbDown = 0;    
                        }
                        var ts = currentFrame.Timestamp - timeStampTakeOff;
                        //Console.WriteLine(ts);
                        if (ts > 1000000)
                        {
                            timeStampTakeOff = currentFrame.Timestamp;
                            if (!this.isFlying)
                            {
                                this.isFlying = true;
                                Console.WriteLine("Take off");
                            }
                        }
                        isVerticalGesture = true;
                    }
                    if (nbDown > 5)
                    {
                        lock (verrou)
                        {
                            nbUp = 0;
                            nbDown = 0;
                        }
                        var ts = currentFrame.Timestamp - timeStampLanding;
                        //Console.WriteLine(ts);
                        if (ts > 1000000)
                        {
                            timeStampLanding = currentFrame.Timestamp;
                            
                            this.isFlying = false;
                            Console.WriteLine("Landing");
                        }
                        isVerticalGesture = true;
                    }
                    if (this.isFlying && !isVerticalGesture)
                    {
                        var dt = DateTime.Now;
                        if (lastNotification.HasValue)
                        {
                            var ts = dt - lastNotification.Value;
                            if (ts.TotalMilliseconds < 50) return;
                        }
                        lastNotification = dt;

                        var position = hand.PalmPosition;
                        var pitch = position.Pitch;
                        var roll = position.Roll;
                        var yaw = position.Yaw;

                        if (!previousPitch.HasValue)
                        {
                            previousPitch = pitch;
                            previousRoll = roll;
                            previousYaw = yaw;
                        }

                        Console.WriteLine("Yaw : {0} - Pitch : {1} - Rool : {2}", yaw, pitch, roll);
                    }
                }
            }
        }
    }
}
